from .django import django
from .blank import blank
from .pypi import pypi
from .flaskmng import flaskmng

templates = {
    "flaskmng":flaskmng,
    "django": django,
    "blank":blank,
    "pypi":pypi
}



from ..package import MYPReader

def addScript(name: str, command: str, description: str = ""):
    myp = MYPReader()
    myp.add_script(
        name=name,
        command=command,
        description=description,
    )
    myp.write()
    print('Script added successfully.')


def removeScript(name: str):
    myp = MYPReader()
    myp.remove_script(name)
    myp.write()
    print('Script removed successfully.')



import json
from typing import List, Any
from .. import const
from pipfile import Pipfile

class MYPReader:
    data: dict
    file_name: str

    def __init__(self, file_name: str = const.filename):
        with open(file_name) as f:
            data: dict = json.load(f)
        self.data = data
        self.file_name = file_name

    def write(self) -> None:
        """Write object instance to `myp.json`"""
        with open(self.file_name, 'wt') as f:
            json.dump(self.data, f, indent=4, sort_keys=True)

    def get_data(self, key: str) -> Any:
        """Get data from object"""
        return self.data[key]

    def set_data(self, key: str, value: Any) -> None:
        """Set data to object"""
        self.data[key] = value

    def add_script(self, name: str, command: str, description: str) -> str:
        """Add script to object"""
        self.data["scripts"][name] = {
            "command": command,
            "description": description,
        }
        return self.data["scripts"][name]

    def remove_script(self, name: str) -> str:
        """Remove script from object"""
        return self.data["scripts"].pop(name)

    def get_dependencies(self, scope: str = "default") -> List[str]:
        """Get dependencies of object"""
        deps: dict = Pipfile.load(filename="./Pipfile").data
        if scope == "default":
            return list(deps["default"])
        elif scope == "dev":
            return list(deps["develop"])
        return []



def pypi_oninit():
    print("Thanks for using pypi template!!!")


pypi = {
    "body": {
        "scripts": {
            "predeploy": {
                "command": "python setup.py sdist bdist_wheel",
                "description": "Build package"
            },
            "deploy": {
                "command": "python -m twine upload --skip-existing --repository-url https://upload.pypi.org/legacy/ dist/*",
                "description": "Upload to PYPI"
            }
        }
    },
    "oninit": pypi_oninit
}



from . import const
import json
import os
import subprocess
import click
from .package import MYPReader
from termcolor2 import c
from PyInquirer import prompt
from typing import Any, List


def loadScripts() -> dict:
    myp = MYPReader()
    return myp.get_data("scripts")


def runScriptDirectly(script: str):
    click.echo('\n\t' + c('>').blue + c('>').yellow + f' {script}\n')
    myp_obj: MYPReader = MYPReader()
    output: subprocess.CompletedProcess
    if myp_obj.get_data("use_environment"):
        # os.system(f'snakenv {myp_obj.get_data("environment")} -c "{script}"')
        output = subprocess.run(['pipenv','run', *script.split()])
    else:
        # os.system(script)
        output = subprocess.run(script.split())
    

def runScriptIfExist(name: str):
    scripts = loadScripts()
    called_script = scripts.get(name, None)
    if called_script:
        cmd = called_script["command"]
        runScriptDirectly(cmd)
    else:
        pass


def runScript(name: str, unknown_options: dict = {}):
    scripts = loadScripts()
    called_script = scripts.get(name, None)
    if called_script:
        cmd = called_script["command"]
        runScriptDirectly(cmd.format(**unknown_options))
    else:
        raise click.ClickException(f'Can not find script named "{name}"')


def take_input(text):
    questions = [
        {
            'type': 'input',
            'name': 'data',
            'message': text
        }
    ]

    answers = prompt(questions)
    return answers['data']


def process_unknown_options(args: List[str]) -> dict:
    data: dict = {}
    temp: str = ""
    for arg in args:
        if arg.startswith("--"):
            key, value = arg[2:].split("=")
            data[key] = value
        elif arg.startswith("-"):
            temp = arg[1:]
        else:
            if temp != "":
                data[temp] = arg
                temp = ""
    return data


def pip_cmd(cmd: str) -> str:
    import sys
    return f"python -m pip {cmd}"


def parse_requirements(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line.split('==')[0] for line in lineiter if line and not line.startswith("#")]
